@@ -5,7 +5,7 @@ use warnings;
 package Path::Tiny;
 # ABSTRACT: File path utility
 
-our $VERSION = '0.064'; # TRIAL
+our $VERSION = '0.066'; # TRIAL
 
 # Dependencies
 use Config;
@@ -772,16 +772,22 @@ sub is_dir { -d $_[0]->[PATH] }
 #pod
 #pod     $fh = path("/tmp/foo.txt")->filehandle($mode, $binmode);
 #pod     $fh = path("/tmp/foo.txt")->filehandle({ locked => 1 }, $mode, $binmode);
+#pod     $fh = path("/tmp/foo.txt")->filehandle({ exclusive => 1  }, $mode, $binmode);
 #pod
 #pod Returns an open file handle.  The C<$mode> argument must be a Perl-style
 #pod read/write mode string ("<" ,">", "<<", etc.).  If a C<$binmode>
 #pod is given, it is set during the C<open> call.
 #pod
-#pod An optional hash reference may be used to pass options.  The only option is
-#pod C<locked>.  If true, handles opened for writing, appending or read-write are
-#pod locked with C<LOCK_EX>; otherwise, they are locked with C<LOCK_SH>.  When using
-#pod C<locked>, ">" or "+>" modes will delay truncation until after the lock is
-#pod acquired.
+#pod An optional hash reference may be used to pass options.
+#pod
+#pod The C<locked> option governs file locking; if true, handles opened for writing,
+#pod appending or read-write are locked with C<LOCK_EX>; otherwise, they are
+#pod locked with C<LOCK_SH>.  When using C<locked>, ">" or "+>" modes will delay
+#pod truncation until after the lock is acquired.
+#pod
+#pod The C<exclusive> option causes the open() call to fail if the file already
+#pod exists.  This corresponds to the O_EXCL flag to sysopen / open(2).
+#pod C<exclusive> implies C<locked> and will set it for you if you forget it.
 #pod
 #pod See C<openr>, C<openw>, C<openrw>, and C<opena> for sugar.
 #pod
@@ -795,7 +801,8 @@ sub is_dir { -d $_[0]->[PATH] }
 sub filehandle {
     my ( $self, @args ) = @_;
     my $args = ( @args && ref $args[0] eq 'HASH' ) ? shift @args : {};
-    $args = _get_args( $args, qw/locked/ );
+    $args = _get_args( $args, qw/locked exclusive/ );
+    $args->{locked} = 1 if $args->{exclusive};
     my ( $opentype, $binmode ) = @args;
 
     $opentype = "<" unless defined $opentype;
@@ -814,6 +821,7 @@ sub filehandle {
             # sysopen in write mode without truncation
             my $flags = $opentype eq ">" ? Fcntl::O_WRONLY() : Fcntl::O_RDWR();
             $flags |= Fcntl::O_CREAT();
+            $flags |= Fcntl::O_EXCL() if $args->{exclusive};
             sysopen( $fh, $self->[PATH], $flags ) or $self->_throw("sysopen");
 
             # fix up the binmode since sysopen() can't specify layers like
@@ -993,11 +1001,17 @@ sub iterator {
 #pod     @contents = path("/tmp/foo.txt")->lines( { chomp => 1, count => 4 } );
 #pod
 #pod Returns a list of lines from a file.  Optionally takes a hash-reference of
-#pod options.  Valid options are C<binmode>, C<count> and C<chomp>.  If C<binmode>
-#pod is provided, it will be set on the handle prior to reading.  If C<count> is
-#pod provided, up to that many lines will be returned. If C<chomp> is set, any
-#pod end-of-line character sequences (C<CR>, C<CRLF>, or C<LF>) will be removed
-#pod from the lines returned.
+#pod options.  Valid options are C<binmode>, C<count> and C<chomp>.
+#pod
+#pod If C<binmode> is provided, it will be set on the handle prior to reading.
+#pod
+#pod If a positive C<count> is provided, that many lines will be returned from the
+#pod start of the file.  If a negative C<count> is provided, the entire file will be
+#pod read, but only C<abs(count)> will be kept and returned.  If C<abs(count)>
+#pod exceeds the number of lines in the file, all lines will be returned.
+#pod
+#pod If C<chomp> is set, any end-of-line character sequences (C<CR>, C<CRLF>, or
+#pod C<LF>) will be removed from the lines returned.
 #pod
 #pod Because the return is a list, C<lines> in scalar context will return the number
 #pod of lines (and throw away the data).
@@ -1014,7 +1028,7 @@ sub iterator {
 #pod intensive.  If memory use is a concern, consider C<openr_utf8> and
 #pod iterating directly on the handle.
 #pod
-#pod Current API available since 0.048.
+#pod Current API available since 0.065.
 #pod
 #pod =cut
 
@@ -1027,12 +1041,18 @@ sub lines {
     my $chomp = $args->{chomp};
     # XXX more efficient to read @lines then chomp(@lines) vs map?
     if ( $args->{count} ) {
-        my ( @result, $counter );
+        my ( $counter, $mod, @result ) = ( 0, abs( $args->{count} ) );
         while ( my $line = <$fh> ) {
             $line =~ s/(?:\x{0d}?\x{0a}|\x{0d})$// if $chomp;
-            push @result, $line;
-            last if ++$counter == $args->{count};
+            $result[ $counter++ ] = $line;
+            # for positive count, terminate after right number of lines
+            last if $counter == $args->{count};
+            # for negative count, eventually wrap around in the result array
+            $counter %= $mod;
         }
+        # reorder results if full and wrapped somewhere in the middle
+        splice( @result, 0, 0, splice( @result, $counter ) )
+          if @result == $mod && $counter % $mod;
         return @result;
     }
     elsif ($chomp) {
@@ -1476,7 +1496,7 @@ sub spew {
     # get default binmode from caller's lexical scope (see "perldoc open")
     $binmode = ( ( caller(0) )[10] || {} )->{'open>'} unless defined $binmode;
     my $temp = path( $self->[PATH] . $$ . int( rand( 2**31 ) ) );
-    my $fh = $temp->filehandle( ">", $binmode );
+    my $fh = $temp->filehandle( { exclusive => 1, locked => 1 }, ">", $binmode );
     print {$fh} map { ref eq 'ARRAY' ? @$_ : $_ } @data;
     close $fh or $self->_throw( 'close', $temp->[PATH] );
 
@@ -1673,7 +1693,7 @@ sub touchpath {
 #pod C<$_> global variable:
 #pod
 #pod     # print paths matching /foo/
-#pod     path("/tmp")->visit( { say if /foo/ }, { recurse => 1} );
+#pod     path("/tmp")->visit( sub { say if /foo/ }, { recurse => 1} );
 #pod
 #pod If the callback returns a B<reference> to a false scalar value, iteration will
 #pod terminate.  This is not the same as "pruning" a directory search; this just
@@ -1761,7 +1781,7 @@ Path::Tiny - File path utility
 
 =head1 VERSION
 
-version 0.064
+version 0.066
 
 =head1 SYNOPSIS
 
@@ -1789,6 +1809,7 @@ version 0.064
   @lines = $file->lines_utf8;
 
   ($head) = $file->lines( {count => 1} );
+  ($tail) = $file->lines( {count => -1} );
 
   # writing files
 
@@ -2144,16 +2165,22 @@ Current API available since 0.053.
 
     $fh = path("/tmp/foo.txt")->filehandle($mode, $binmode);
     $fh = path("/tmp/foo.txt")->filehandle({ locked => 1 }, $mode, $binmode);
+    $fh = path("/tmp/foo.txt")->filehandle({ exclusive => 1  }, $mode, $binmode);
 
 Returns an open file handle.  The C<$mode> argument must be a Perl-style
 read/write mode string ("<" ,">", "<<", etc.).  If a C<$binmode>
 is given, it is set during the C<open> call.
 
-An optional hash reference may be used to pass options.  The only option is
-C<locked>.  If true, handles opened for writing, appending or read-write are
-locked with C<LOCK_EX>; otherwise, they are locked with C<LOCK_SH>.  When using
-C<locked>, ">" or "+>" modes will delay truncation until after the lock is
-acquired.
+An optional hash reference may be used to pass options.
+
+The C<locked> option governs file locking; if true, handles opened for writing,
+appending or read-write are locked with C<LOCK_EX>; otherwise, they are
+locked with C<LOCK_SH>.  When using C<locked>, ">" or "+>" modes will delay
+truncation until after the lock is acquired.
+
+The C<exclusive> option causes the open() call to fail if the file already
+exists.  This corresponds to the O_EXCL flag to sysopen / open(2).
+C<exclusive> implies C<locked> and will set it for you if you forget it.
 
 See C<openr>, C<openw>, C<openrw>, and C<opena> for sugar.
 
@@ -2230,11 +2257,17 @@ Current API available since 0.016.
     @contents = path("/tmp/foo.txt")->lines( { chomp => 1, count => 4 } );
 
 Returns a list of lines from a file.  Optionally takes a hash-reference of
-options.  Valid options are C<binmode>, C<count> and C<chomp>.  If C<binmode>
-is provided, it will be set on the handle prior to reading.  If C<count> is
-provided, up to that many lines will be returned. If C<chomp> is set, any
-end-of-line character sequences (C<CR>, C<CRLF>, or C<LF>) will be removed
-from the lines returned.
+options.  Valid options are C<binmode>, C<count> and C<chomp>.
+
+If C<binmode> is provided, it will be set on the handle prior to reading.
+
+If a positive C<count> is provided, that many lines will be returned from the
+start of the file.  If a negative C<count> is provided, the entire file will be
+read, but only C<abs(count)> will be kept and returned.  If C<abs(count)>
+exceeds the number of lines in the file, all lines will be returned.
+
+If C<chomp> is set, any end-of-line character sequences (C<CR>, C<CRLF>, or
+C<LF>) will be removed from the lines returned.
 
 Because the return is a list, C<lines> in scalar context will return the number
 of lines (and throw away the data).
@@ -2251,7 +2284,7 @@ actually faster than relying on C<:encoding(UTF-8)>, though a bit memory
 intensive.  If memory use is a concern, consider C<openr_utf8> and
 iterating directly on the handle.
 
-Current API available since 0.048.
+Current API available since 0.065.
 
 =head2 mkpath
 
@@ -2546,7 +2579,7 @@ For convenience, the C<Path::Tiny> object will also be locally aliased as the
 C<$_> global variable:
 
     # print paths matching /foo/
-    path("/tmp")->visit( { say if /foo/ }, { recurse => 1} );
+    path("/tmp")->visit( sub { say if /foo/ }, { recurse => 1} );
 
 If the callback returns a B<reference> to a false scalar value, iteration will
 terminate.  This is not the same as "pruning" a directory search; this just
@@ -2749,7 +2782,7 @@ David Golden <dagolden@cpan.org>
 
 =head1 CONTRIBUTORS
 
-=for stopwords Alex Efros Chris Williams David Steinbrunner Doug Bell Gabor Szabo Gabriel Andrade George Hartzell Geraud Continsouzas Goro Fuji Graham Knop Karen Etheridge Martin Kjeldsen Michael G. Schwern regina-verbae Smylers Toby Inkster 김도형 - Keedi Kim
+=for stopwords Alex Efros Chris Williams David Steinbrunner Doug Bell Gabor Szabo Gabriel Andrade George Hartzell Geraud Continsouzas Goro Fuji Graham Knop James Hunt Karen Etheridge Martin Kjeldsen Michael G. Schwern regina-verbae Smylers Toby Inkster 김도형 - Keedi Kim
 
 =over 4
 
@@ -2795,6 +2828,10 @@ Graham Knop <haarg@haarg.org>
 
 =item *
 
+James Hunt <james@niftylogic.com>
+
+=item *
+
 Karen Etheridge <ether@cpan.org>
 
 =item *
